/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is cookiebutton.
 *
 * The Initial Developer of the Original Code is
 * Pang Lih Wuei (basic@mozdev.org).
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var cookiePermission =
{
	clearWhenDeny      : false,
	prefObserver       : 
	{
		observe        : function (subject, topic, data) {this._cookie.update();},
		QueryInterface : function (aIID) 
		{
			if (aIID.equals(Components.interfaces.nsIObserver) ||
				aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
				aIID.equals(Components.interfaces.nsISupports))
					return this;
			throw Components.results.NS_NOINTERFACE;
		}
	},
	extPrefObserver   :
	{
		observe        : function (subject, topic, data)
		{
			this._update();
		},
		QueryInterface : function (aIID) 
		{
			if (aIID.equals(Components.interfaces.nsIObserver) ||
				aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
				aIID.equals(Components.interfaces.nsISupports))
					return this;
			throw Components.results.NS_NOINTERFACE;
		},
		_update : function ()
		{
			var cookie = this._cookie;
			var branch = cookie.extPrefBranch;
			cookie.clearWhenDeny = branch.getBoolPref("clear-when-deny");
			var clear = cookie.clearElement;
			if (branch.getBoolPref("show-clear-cookies"))
				clear.removeAttribute("hidden");
			else
				clear.setAttribute("hidden", "true");
		}
		
	},
	locationListener  :
	{
		onLocationChange    : function (w, r, l)         {this._cookie.update(); return true;},
		// to avoid error messages and/or make firefox function properly, sigh, NOTIFY_LOCATION seems to notify more than location stuff
		onProgressChange    : function (w,r,cS,mS,cT,mT) {return true;},
		onSecurityChange    : function (w,r,s)           {return true;},
		onStateChange       : function (w,r,f,s)         {return true;},
		onStatusChange      : function (w,r,s,m)         {return true;},
		onLinkIconAvailable : function (href)            {return true;}

		// should not need update()s here, but we seem to be hitting a firebird bug where
		// onLocationChange sometimes fail to trigger
		// 2005: disabling, seems like the bug was fix
		//onProgressChange    : function (w,r,cS,mS,cT,mT) {this._cookie.update();return true;},
		//onSecurityChange    : function (w,r,s)           {this._cookie.update();return true;},
		//onStateChange       : function (w,r,f,s)         {this._cookie.update();return true;},
		//onStatusChange      : function (w,r,s,m)         {this._cookie.update();return true;},
		//onLinkIconAvailable : function (href)            {this._cookie.update();return true;}
	},
	init : function (doc, atype, disable, enable) 
	{

		var classes = Components.classes;
		var ifaces  = Components.interfaces;
		this.clearElement      = doc.getElementById("cookie-perm-"+atype+"-clear");
		this.strings           = doc.getElementById("cookie-perm-"+atype+"-strings");
		this.browser           = doc.getElementById("content");
		this.disableElement    = disable;
		this.enableElement     = enable;
		this.prefService       = classes["@mozilla.org/preferences;1"].getService(ifaces.nsIPrefService);
		this.consoleService    = classes['@mozilla.org/consoleservice;1'].getService(ifaces.nsIConsoleService);
		this.permissionManager = classes["@mozilla.org/permissionmanager;1"].getService(ifaces.nsIPermissionManager);
		this.cookieManager     = classes["@mozilla.org/cookiemanager;1"].getService(ifaces.nsICookieManager);
		this.nsICookie         = ifaces.nsICookie;
		this.extPrefBranch     = this.prefService.getBranch("extensions.cookiebutton.").QueryInterface(ifaces.nsIPrefBranch).QueryInterface(ifaces.nsIPrefBranchInternal);
		this.prefBranch        = this.prefService.getBranch("network.cookie.").QueryInterface(ifaces.nsIPrefBranch).QueryInterface(ifaces.nsIPrefBranchInternal);

		// initialize observers/listeners
		this.prefObserver._cookie = this;
		this.extPrefObserver._cookie = this;
		this.locationListener._cookie = this;

		// observe cookie prefs
		this.prefBranch.addObserver("cookieBehavior", this.prefObserver,true);
// no longer used
//			this.prefBranch.addObserver("warnAboutCookies", this.prefObserver,true);

		// observe life time policy of cookies
		this.prefBranch.addObserver("lifetimePolicy", this.prefObserver,true);

		this.extPrefBranch.addObserver("clear-when-deny", this.extPrefObserver, true);
		this.extPrefBranch.addObserver("show-clear-cookies", this.extPrefObserver, true);
		
		// setup browser location webProgressListener
		this.browser.addProgressListener(this.locationListener,ifaces.nsIWebProgress.NOTIFY_LOCATION);

		this.extPrefObserver._update();
		this.update();
	},
	// update the button
	update : function ()
	{
		var location = this.browser.currentURI;
		// check if we are http or https, if not disable button
		if(!location || (location.scheme.search(/^https?$/) == -1))
		{
			this.disableElement();
			return false;
		}
		this.enableElement(location);
		return true;
	},
	// get default premission from preferences
	getDefaultPref: function ()
	{
		var behavior = 0;

		// for some earlier firebird builds
		//try { if(!this.prefBranch.getBoolPref("enable")) return "default-deny";	}
		//catch (e) { }
		//try { if(this.prefBranch.getBoolPref("enableForOriginatingWebsiteOnly")) behavior = 1; }
		//catch (e)
		//{
			// hopefully there will be no more changes to the cookie pref
			try { behavior = this.prefBranch.getIntPref("cookieBehavior"); }
			catch (e) {	this.consoleService.logStringMessage("cookiebutton could not read network.cookie.cookieBehavior pref"); }
		//}

		// cookies disabled
		if(behavior == 2) return "default-deny";

		// warn about cookies -- no longer in use
//			try { if(this.prefBranch.getBoolPref("warnAboutCookies")) return "default-warn"; }
//			catch (e) { dump("can't read network.cookie.warnAboutCookies");	}

		try
		{
			switch (this.prefBranch.getIntPref("lifetimePolicy"))
			{
			case 1: return "default-warn";
			case 2: return behavior == 0 ? "default-third-party-session" : "default-session";
			}
		}
		catch (e) { this.consoleService.logStringMessage("cookiebutton could not read network.cookie.lifetimePolicy pref");	}
		
		switch (behavior)
		{
		case 0: return "default-third-party"; // enable third party cookies
		case 1: return "default-allow";       // enable cookies
//			case 3:	return "default-p3p";         // base on p3p
		}
		return "default"; // should never happen
	},
	// check permission
	getPermission : function(location)
	{
		switch(this.permissionManager.testPermission(location,"cookie"))
		{
		case this.permissionManager.ALLOW_ACTION: return "allow";
		case this.permissionManager.DENY_ACTION:  return "deny";
		// constant from nsCookiePermission.cpp
		// ALLOW_SESSION = 8
		case 8:                    return "allow-session";
		default:                   return this.getDefaultPref();
		}
	},
	// set permission
	setPermission : function (option)
	{
		var location = this.browser.currentURI;
		this.permissionManager.remove(location.host,"cookie");
		switch(option)
		{
		case "allow-session": this.permissionManager.add(location,"cookie",8);   break;
		case "allow":         this.permissionManager.add(location,"cookie",this.permissionManager.ALLOW_ACTION); break;
		case "deny":
			this.permissionManager.add(location,"cookie",this.permissionManager.DENY_ACTION);
			// fall through
		case "default-deny":
			if (this.clearWhenDeny) this.clearCookies();
			break;
		}
		this.update();
	},
	clearCookies : function ()
	{
		var host = this.browser.currentURI.host;
		var cMan = this.cookieManager;
		var iter = cMan.enumerator;
		var nsICookie = this.nsICookie;
		while(iter.hasMoreElements())
		{
			var cookie = iter.getNext().QueryInterface(nsICookie);
			if (host == cookie.host) cMan.remove(cookie.host, cookie.name, cookie.path, false);
		}
//		this.consoleService.logStringMessage("cleared cookies for " + host);
	},
	handleMenuCommand : function (menuitem)
	{
		if (menuitem.value != '')
			this.setPermission(menuitem.value);
		else
			this.clearCookies();
	},
	// generate label from value and location
	labelGenerator : function (type,location)
	{
		switch(type)
		{
			case "default-session":             return this.strings.getString("default.session");
			case "default-third-party-session": return this.strings.getString("default.3rdPartySession");
			case "default-third-party":         return this.strings.getString("default.3rdParty");
			case "default-allow":               return this.strings.getString("default.accept");
			case "default-deny":                return this.strings.getString("default.reject");
			case "default-warn":                return this.strings.getString("default.warn");
//				case "default-p3p":                 return "Default (Based on P3P settings)";
// on firefox 1.0.x at least, getFormattedString is buggy (only grabs the first character) so we use this slower replace hack
			case "allow-session":               return this.strings.getString("host.session").replace(/%s/,location.host);
			case "allow":                       return this.strings.getString("host.accept").replace(/%s/,location.host);
			case "deny":                        return this.strings.getString("host.reject").replace(/%s/,location.host);
		}
		return "";
	},
	// update dropdown menu
	updatePermMenu : function (items)
	{
		// should not need this here, but we seem to be hitting a firebird bug where
		// the webProgressListener sometimes fail to trigger
		// 2005 : think this bug is fixed now.
		// if(!this.update()) return false;
		var location = this.browser.currentURI;

		var value = this.getPermission(location);
		var default_value = (value.substr(0,7)=="default") ? value : this.getDefaultPref();

		// "default" option
		items[0].value = default_value;

		// set popup/dropdown label/values
		var last = items.length - 1;
		for (var i = 0; i < last; i++)
		{

			var option = items[i].value;
			if (option == value) items[i].setAttribute("checked","true");
			items[i].label = this.labelGenerator(option,location);
		}
		// clear cookies menuitem
// on firefox 1.0.x at least, getFormattedString is buggy (only grabs the first character) so we use this slower replace hack
		items[last].label = this.strings.getString("host.clear").replace(/%s/,location.host);
		return true;
	}
};

// */